"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Retrieves all primitive value and their corresponding paths from an object
 *
 * The order of the returned values is not guarenteed to be sorted in any way
 *
 * @param object the object to retrieve all primitives values from
 * @return an array of all primitive values and their corresponding paths
 *
 * @example
 * ```javascript
 * const object = { foo: { bar: "bar", baz: [true, 0] }}
 * const result = values(object)
 * // result === [
 *   { path: [ "foo", "bar", ], value: "bar" },
 *   { path: [ "foo", "baz", 0 ], value: true },
 *   { path: [ "foo", "baz", 1 ], value: 0 }
 * ]
 * ```
 */
function valuesAndPaths(object) {
    return _valuesAndPaths(object, []);
}
exports.valuesAndPaths = valuesAndPaths;
/**
 * @private
 * @hidden
 */
function _valuesAndPaths(object, prefix) {
    if (null === object || typeof object !== "object") {
        return [{ path: prefix, value: object }];
    }
    else {
        if (object instanceof Array) {
            return object
                .map(function (element, i) { return _valuesAndPaths(element, prefix.concat([i])); })
                .reduce(function (a, b) { return a.concat(b); }, []);
        }
        else {
            return Object.keys(object)
                .map(function (key) {
                return _valuesAndPaths(object[key], prefix.concat([key]));
            })
                .reduce(function (a, b) { return a.concat(b); }, []);
        }
    }
}
exports.default = valuesAndPaths;
//# sourceMappingURL=valuesAndPaths.js.map